@@ -277,7 +277,7 @@ sub unlog4j {
 
     $string =~ s#^org\.apache\.##;
     $string =~ s#^log4j\.##;
-    $string =~ s#^log4perl\.##;
+    $string =~ s#^log4perl\.##i;
 
     $string =~ s#\.#::#g;
 
