@@ -10,14 +10,14 @@ use Log::Log4perl::Level;
 use Log::Log4perl::Layout;
 use Log::Log4perl::Appender;
 use Log::Dispatch;
-use Data::Dumper;
 use Carp;
+use Carp::Heavy;
 
 use constant DEBUG => 0;
 
     # Initialization
 our $ROOT_LOGGER;
-our $LOGGERS_BY_NAME;
+our $LOGGERS_BY_NAME = {};
 our %APPENDER_BY_NAME = ();
 our $INITIALIZED;
 
@@ -34,11 +34,25 @@ __PACKAGE__->reset();
 sub reset {
 ##################################################
     $ROOT_LOGGER        = __PACKAGE__->_new("", $DEBUG);
-    $LOGGERS_BY_NAME    = {};
+#    $LOGGERS_BY_NAME    = {};  #leave this alone, it's used by 
+                                #reset_all_output_methods when the config changes
     %APPENDER_BY_NAME   = ();
     $DISPATCHER         = Log::Dispatch->new();
     undef $INITIALIZED;
     Log::Log4perl::Appender::reset();
+
+    #clear out all the existing appenders
+    foreach my $logger (values %$LOGGERS_BY_NAME){
+        $logger->{appender_names} = ();
+
+	#this next bit deals with an init_and_watch case where a category
+	#is deleted from the config file, we need to zero out the existing
+	#loggers so ones not in the config file not continue with their old
+	#behavior --kg
+        next if $logger eq $ROOT_LOGGER;
+        $logger->{level} = undef;
+        $logger->level();  #set it from the heirarchy
+    }
 }
 
 ##################################################
@@ -133,8 +147,11 @@ sub set_output_methods {
 
     my %priority = %Log::Log4perl::Level::PRIORITY; #convenience and cvs
 
+   # changed to >= from <= as level ints were reversed
     foreach my $levelname (keys %priority){
-        if ($priority{$levelname} <= $level) {
+        if (Log::Log4perl::Level::isGreaterOrEqual($level,
+						   $priority{$levelname}
+						   )) {
             print "  ($priority{$levelname} <= $level)\n"
                   if DEBUG;
             $self->{$levelname} = $coderef;
@@ -143,9 +160,9 @@ sub set_output_methods {
             $self->{$levelname} = $noop;
         }
 
-        print("  Setting $self->{category}.$levelname to ",
+        print("  Setting [$self] $self->{category}.$levelname to ",
               ($self->{$levelname} == $noop ? "NOOP" : 
-              ("Coderef: " . scalar @appenders . " appenders")), 
+              ("Coderef [$coderef]: " . scalar @appenders . " appenders")), 
               "\n") if DEBUG;
     }
 }
@@ -186,7 +203,6 @@ sub generate_coderef {
 
           print("  Sending message '\$message' (\$level) " .
                 "to \$appender_name\n") if DEBUG;
-    
           \$appender->log(
               #these get passed through to Log::Dispatch
               { name    => \$appender_name,
@@ -245,27 +261,27 @@ sub generate_watch_code {
 
     return <<'EOL';
         print "exe_watch_code:\n" if DEBUG;
+                       
+        # more closures here
+        if ( ($LAST_CHECKED_AT + $WATCH_DELAY) < time()){
+        
+             $LAST_CHECKED_AT = time();
 
-        #more closures here
-        if ( (($LAST_CHECKED_AT + $WATCH_DELAY) < time())
-                &&  ($LAST_CHANGED_AT < (stat($FILE_TO_WATCH))[9] )){
-                
-            print "  Config file has been modified\n" if DEBUG;
-
-            %APPENDER_BY_NAME = ();
-            $DISPATCHER = Log::Dispatch->new();
-            
-            Log::Log4perl->init_and_watch($FILE_TO_WATCH, $WATCH_DELAY);
-            
-            my $methodname = lc($level);
-            $logger->$methodname($message); # send the message 
-                                            # to the new configuration
-            
-            $LAST_CHECKED_AT = time();
-            
-            return;
-        }else{
-            $LAST_CHECKED_AT = time();
+             print "  Checking $FILE_TO_WATCH for changes ...\n" if DEBUG;
+        
+             if ($LAST_CHANGED_AT < (stat($FILE_TO_WATCH))[9] ){
+                       
+                 $LAST_CHANGED_AT = (stat(_))[9];
+                       
+                 print "  Config file has been modified\n" if DEBUG;
+                       
+                 Log::Log4perl->init_and_watch($FILE_TO_WATCH, $WATCH_DELAY);
+                       
+                 my $methodname = lc($level);
+                 $logger->$methodname($message); # send the message
+                                                 # to the new configuration
+             }
+             return;
         }
 EOL
 }
@@ -461,6 +477,8 @@ sub log {
 ##################################################
     my ($self, $priority, @messages) = @_;
 
+    confess("log: No priority given!") unless defined($priority);
+
        # Just in case of 'init_and_watch' -- see Changes 0.21
     $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
 
@@ -474,60 +492,322 @@ sub log {
                     Log::Log4perl::Level::to_level($priority));
 }
 
+######################################################################
+#
+# create_custom_level 
+# creates a custom level
+# in theory, could be used to create the default ones
+
+sub create_custom_level {
+  my $level = shift || die("create_custom_level: forgot to pass in a level string!");
+  my $after = shift || die("create_custom_level: forgot to pass in a level after which to place the new level!");
+  my $syslog_equiv = shift; # can be undef
+
+  ## only let users create custom levels before initialization
+
+  die("create_custom_level must be called before init or first get_logger() call") if ($INITIALIZED);
+
+  my %PRIORITY = %Log::Log4perl::Level::PRIORITY; #convenience
+
+  die("create_custom_level: no such level \"$after\"! Use one of: ", join(", ", sort keys %PRIORITY))
+    unless $PRIORITY{$after};
+
+  # figure out new int value by AFTER + (AFTER+ 1) / 2
+
+  my $next_prio = Log::Log4perl::Level::get_lower_level($PRIORITY{$after}, 1);
+  my $cust_prio = int(($PRIORITY{$after} + $next_prio) / 2);
+
+#   CORE::warn("Creating prio $cust_prio between $PRIORITY{$after} and $next_prio");
+
+  die(qq{create_custom_level: Calculated level of $cust_prio already exists!
+      This should only happen if you've made some insane number of custom
+      levels (like 15 one after another)
+      You can usually fix this by re-arranging your code from:
+      create_custom_level("cust1", X);
+      create_custom_level("cust2", X);
+      create_custom_level("cust3", X);
+      create_custom_level("cust4", X);
+      create_custom_level("cust5", X);
+      into:
+      create_custom_level("cust3", X);
+      create_custom_level("cust5", X);
+      create_custom_level("cust4", 4);
+      create_custom_level("cust2", cust3);
+      create_custom_level("cust1", cust2);
+   }) if (${Log::Log4perl::Level::LEVELS{$cust_prio}});
+
+  Log::Log4perl::Level::add_priority($level, $cust_prio, $syslog_equiv);
+
+  print("Adding prio $level at $cust_prio\n") if DEBUG;
+
+  # get $LEVEL into namespace of Log::Log4perl::Logger to 
+  # create $logger->foo nd $logger->is_foo
+  my $name = "Log::Log4perl::Logger::";
+  my $key = $level;
+
+  no strict qw(refs);
+  # be sure to use ${Log...} as CVS adds log entries for Log
+  *{"$name$key"} = \${Log::Log4perl::Level::PRIORITY{$level}};
+
+  # now, stick it in the caller's namespace
+  $name = caller(0) . "::";
+  *{"$name$key"} = \${Log::Log4perl::Level::PRIORITY{$level}};
+  use strict qw(refs);
+
+  create_log_level_methods($level);
+
+  return 0;
+
+}
+
+########################################
+#
+# if we were hackin' lisp (or scheme), we'd be returning some lambda
+# expressions. But we aren't. :) So we'll just create some strings and
+# eval them.
+sub create_log_level_methods {
+  my $level = shift || die("create_log_level_methods: forgot to pass in a level string!");
+  my $lclevel = lc($level);
+  my $levelint = uc($level) . "_INT";
+
+  no strict qw(refs);
+
+  # This is a bit better way to create code on the fly than eval'ing strings.
+  # -erik
+
+  *{__PACKAGE__ . "::$lclevel"} = sub {
+        print "$lclevel: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+        init_warn() unless $INITIALIZED;
+        $_[0]->{$level}(@_, $level);
+     };
+
+  *{__PACKAGE__ . "::is_$lclevel"} = sub { 
+    return Log::Log4perl::Level::isGreaterOrEqual($_[0]->level(),
+						  $$level
+						  ); 
+  };
+  
+  use strict qw(refs);
+
+  return 0;
+
+}
+
+#now lets autogenerate the logger subs based on the defined priorities
+foreach my $level (keys %Log::Log4perl::Level::PRIORITY){
+  create_log_level_methods($level);
+}
+
 ##################################################
 #expected args are $logger, $msg, $levelname
 
-sub fatal {
-   print "fatal: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
-       # Just in case of 'init_and_watch' -- see Changes 0.21
-   $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
-   init_warn() unless $INITIALIZED;
-   $_[0]->{FATAL}(@_, 'FATAL');
-}
+#sub fatal {
+#   print "fatal: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+#   init_warn() unless $INITIALIZED;
+#   $_[0]->{FATAL}(@_, 'FATAL');
+#}
+#
+#sub error {
+#   print "error: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+#   init_warn() unless $INITIALIZED;
+#   $_[0]->{ERROR}(@_, 'ERROR');
+#}
+#
+#sub warn {
+#   print "warn: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+#   init_warn() unless $INITIALIZED;
+#   $_[0]->{WARN} (@_, 'WARN' );
+#}
+#
+#sub info {
+#   print "info: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+#   init_warn() unless $INITIALIZED;
+#   $_[0]->{INFO} (@_, 'INFO' );
+#}
+#
+#sub debug {
+#   print "debug: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
+#   init_warn() unless $INITIALIZED;
+#   $_[0]->{DEBUG}(@_, 'DEBUG');
+#}
 
-sub error {
-   print "error: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
-       # Just in case of 'init_and_watch' -- see Changes 0.21
-   $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
-   init_warn() unless $INITIALIZED;
-   $_[0]->{ERROR}(@_, 'ERROR');
-}
-
-sub warn {
-   print "warn: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
-       # Just in case of 'init_and_watch' -- see Changes 0.21
-   $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
-   init_warn() unless $INITIALIZED;
-   $_[0]->{WARN} (@_, 'WARN' );
-}
-
-sub info {
-   print "info: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
-       # Just in case of 'init_and_watch' -- see Changes 0.21
-   $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
-   init_warn() unless $INITIALIZED;
-   $_[0]->{INFO} (@_, 'INFO' );
-}
-
-sub debug {
-   print "debug: ($_[0]->{category}/$_[0]->{level}) [@_]\n" if DEBUG;
-       # Just in case of 'init_and_watch'
-   $_[0] = $LOGGERS_BY_NAME->{$_[0]->{category}} if defined $LAST_CHECKED_AT;
-   init_warn() unless $INITIALIZED;
-   $_[0]->{DEBUG}(@_, 'DEBUG');
-}
-
-sub is_debug { return $_[0]->level() >= $DEBUG; }
-sub is_info  { return $_[0]->level() >= $INFO; }
-sub is_warn  { return $_[0]->level() >= $WARN; }
-sub is_error { return $_[0]->level() >= $ERROR; }
-sub is_fatal { return $_[0]->level() >= $FATAL; }
+#sub is_debug { return $_[0]->level() >= $DEBUG; }
+#sub is_info  { return $_[0]->level() >= $INFO; }
+#sub is_warn  { return $_[0]->level() >= $WARN; }
+#sub is_error { return $_[0]->level() >= $ERROR; }
+#sub is_fatal { return $_[0]->level() >= $FATAL; }
 sub init_warn {
-    CORE::warn "Seems like no initialization happened. Forgot to call init()?\n";
+    CORE::warn "Log4perl: Seems like no initialization happened. Forgot to call init()?\n";
     # Only tell this once;
     $INITIALIZED = 1;
               }
 
+##################################################
+# call me from a sub-func to spew the sub-func's caller
+sub callerline {
+  # the below could all be just:
+  # my ($pack, $file, $line) = caller(2);
+  # but if we every bury this further, it'll break. So we do this
+  # little trick stolen and paraphrased from Carp/Heavy.pm
+
+  my $i = 0;
+  my (undef, $localfile, undef) = caller($i++);
+  my ($pack, $file, $line);
+  do {
+    ($pack, $file, $line) = caller($i++);
+  } while ($file && $file eq $localfile);
+
+  # now, create the return message
+  my $mess = " at $file line $line";
+  # Someday, we'll use Threads. Really.
+  if (defined &Thread::tid) {
+    my $tid = Thread->self->tid;
+    $mess .= " thread $tid" if $tid;
+  }
+  return (@_, $mess, "\n");
+}
+
+sub and_warn {
+  my $self = shift;
+  my $msg = join("", @_[0 .. $#_]);
+  chomp $msg;
+  CORE::warn(callerline($msg));
+}
+
+sub and_die {
+  my $self = shift;
+  my $msg = join("", @_[0 .. $#_]);
+  chomp $msg;
+  die(callerline($msg));
+}
+
+##################################################
+
+sub logwarn {
+  my $self = shift;
+  if ($self->is_warn()) {
+    $self->warn(@_);
+    $self->and_warn(@_);
+  }
+}
+
+sub logdie {
+  my $self = shift;
+  if ($self->is_fatal()) {
+    $self->fatal(@_);
+  }
+  # no matter what, we die... 'cuz logdie wants you to die.
+  $self->and_die(@_);
+}
+
+##################################################
+
+# for die and warn, carp long/shortmess return line #s and the like
+sub noop {
+  return @_;
+}
+
+##################################################
+
+# clucks and carps are WARN level
+sub logcluck {
+  my $self = shift;
+  if ($self->is_warn()) {
+    my $message = Carp::longmess(@_);
+    foreach (split(/\n/, $message)) {
+      $self->warn("$_\n");
+    }
+    CORE::warn(noop($message));
+  }
+}
+
+sub logcarp {
+  my $self = shift;
+  if ($self->is_warn()) {
+    my $message = Carp::shortmess(@_);
+    foreach (split(/\n/, $message)) {
+      $self->warn("$_\n");
+    }
+    CORE::warn(noop($message));
+  }
+} 
+
+# croaks and confess are FATAL level
+sub logcroak {
+  my $self = shift;
+  my $message = Carp::shortmess(@_);
+  if ($self->is_fatal()) {
+    foreach (split(/\n/, $message)) {
+      $self->fatal("$_\n");
+    }
+  }
+  # again, we die no matter what
+  die(noop($message));
+}
+
+sub logconfess {
+  my $self = shift;
+  my $message = Carp::longmess(@_);
+  if ($self->is_fatal()) {
+    foreach (split(/\n/, $message)) {
+      $self->fatal("$_\n");
+    }
+  }
+  # again, we die no matter what
+  die(noop($message));
+}
+
+##################################################
+# 
+# in case people prefer to use error for warning
+
+sub error_warn {
+  my $self = shift;
+  if ($self->is_error()) {
+    $self->error(@_);
+    $self->and_warn(@_);
+  }
+}
+
+sub error_die {
+  my $self = shift;
+  if ($self->is_error()) {
+    $self->error(@_);
+  }
+  $self->and_die(@_);
+}
+
+sub more_logging {
+  my ($self) = shift;
+  return $self->dec_level(@_);
+}
+
+sub inc_level {
+    my ($self, $delta) = @_;
+
+    $delta ||= 1;
+
+    $self->level(Log::Log4perl::Level::get_higher_level($self->level(), $delta));
+
+    $self->set_output_methods;
+
+}
+
+sub less_logging {
+  my ($self) = shift;
+  return $self->inc_level(@_);
+}
+
+sub dec_level {
+    my ($self, $delta) = @_;
+
+    $delta ||= 1;
+
+    $self->level(Log::Log4perl::Level::get_lower_level($self->level(), $delta));
+
+    $self->set_output_methods;
+}
+
 ##################################################
 
 1;
